﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Linq.Extensions;
using System.Collections.Generic;
using System.ComponentModel;
using WPFTestApp.Data;

namespace PrestImmo.Modules.DataService.Managers.Base
{
    public class GenericQuery<TElement> : IQuery<TElement> where TElement : class, new()
    {
        public GenericQuery()
        {
            this.Predicate = ExpressionExtension.True<TElement>();
        }

        public Expression<Func<TElement, bool>> Predicate { get; protected set; }

        private int totalCount;
        public int TotalCount
        {
            get { return this.totalCount; }
            protected set
            {
                this.totalCount = value;
                this.OnPropertyChanged("TotalCount");
            }
        }

        #region IItemsProvider Members

        public virtual int GetTotal()
        {
            try
            {
                using (var context = new ModelEntities())
                {
                    this.TotalCount = context.CreateObjectSet<TElement>().Count(Predicate);
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine("Query operation 'GetTotal' failed.", ex.ToString());
                this.TotalCount = 0;
            }

            return this.TotalCount;
        }

        public virtual IList<TElement> GetList(int startIndex, int pageCount, out int overallCount)
        {
            overallCount = this.TotalCount;
            IList<TElement> result = null;

            try
            {
                using (var context = new ModelEntities())
                {
                    result = context.CreateObjectSet<TElement>().Where(Predicate).ToList();
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine("Query operation 'GetList' failed.", ex.ToString());
                result = new List<TElement>();
            }

            return result;
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion
    }
}
